

#version 3.8;

/* environment, lights, camera.
 * -------------------------------------------------------------------------- *
 */

global_settings {
  assumed_gamma 1
  max_trace_level 20
#ifdef (radiosity_)
  radiosity {
    count 100
    error_bound .5
    pretrace_end .005
    recursion_limit 2
  }
#end
#ifdef (photons_)
  photons { spacing 0.002 autostop 0 }
#end
}

light_source {<-9.5,10,30>*100 color srgb .65 parallel shadowless}
light_source {<1,1,.1>*100  color srgb .65 parallel shadowless}
light_source {<1,1,.01>*100  color srgb .65 parallel}

camera {
  location <30,7,15>
  right x * (image_width/image_height)
  up y
  angle 48
  look_at <48,1,39>
}



/* height field.
 * -------------------------------------------------------------------------- *
 */

#declare hfx_ = 470 / 5;
#declare hfz_ = 390 / 5;

#declare HF = height_field {
  tga "hfdata.tga"
  pigment {color srgb .65}
  scale <hfx_,4.5,hfz_>
};

object {HF}


/* grass cover.
 * -------------------------------------------------------------------------- *
 * place tufts, one per square unit, with random variation on height and
 * exact placement.  all minus small offset to "plant" firmly.
 */

#macro hfElevation(hf_,res_,x_,z_)
  #local yres_ = res_;
  #local obbl_ = min_extent(hf_);
  #local obbu_ = max_extent(hf_);
//  hf_checkXZ(obbl_,obbu_,x_,z_)
  #local odim_ = obbu_ - obbl_;
  #local llim_ = int(odim_.y * yres_ + .5);
  #local yinc_ = odim_.y / llim_;
#if (0)
hf_emit3V("HF BB lower",obbl_,12)
hf_emit3V("HF BB upper",obbu_,12)
hf_emit3V("HF dimension",odim_,6)
hf_emitVal("resolution",yres_,0)
hf_emitVal("loop limit",llim_,0)
hf_emitVal("Y-axis incr",yinc_,6)
#end
  /* one extra scan/point, should never reach end though */
  #local last_ = <x_,obbu_.y,z_>;
  #local flag_ = 0;
  #for (i_,0,llim_)
    #local y_ = obbu_.y - (i_ * yinc_);
    #if (inside(hf_,<x_,y_,z_>))
      #local flag_ = 1;
      #break
    #end
    #local last_ = <x_,y_,z_>;
  #end
  #if (!flag_)
    #warning concat("oops, scan did not touch HF.")
    #local last_ = <0,0,0>;
  #end
  last_
#end


#declare NTUFTS = 100;

#include "tufts.inc"

#declare offs_ = .05;

#local R = seed(20200905);

#macro rTuft(r_) (int(rand(r_)*NTUFTS)) #end
#macro rA(r_) (rand(r_)*.5-.5) #end
#macro rH(r_) ((13 + rand(r_)*(19-13))/10) #end

union {
#for (j_,0,hfz_-2)
  #for (i_,0,hfx_-2)
    #local xx_ = i_ + .5 + rA(R);
    #local zz_ = j_ + .5 + rA(R);
    #local pt_ = hfElevation(HF,250,xx_,zz_);
    object {
      grasstufts[rTuft(R)]
      scale rH(R)
      translate <pt_.x,pt_.y-offs_,pt_.z>
    }
  #end
#end
}

